/*!\file InputToResultx
 * \brief: average field throfieldh thickness
 */

#include "./InputToResultx.h"

#include "../../shared/shared.h"
#include "../../objects/objects.h"
#include "../../include/include.h"
#include "../../toolkits/toolkits.h"
#include "../../EnumDefinitions/EnumDefinitions.h"

void InputToResultx(Result** presult,DataSet* elements,DataSet* nodes, DataSet* vertices, DataSet* loads, DataSet* materials,Parameters* parameters,int enum_type,int id, double time, int step){

	int i,j,count;

	/*output: */
	Result* result=NULL;

	/*Intermediary output vector*/
	double* patches=NULL; //build a matrix of patch information, corresponding to the input with  name enum_type
	int     patch_numrows,patch_numcols;

	/*First, get elements*/
	elements->Configure(elements,loads, nodes,vertices, materials,parameters);

	/*Figure out size of patches matrix: */
	PatchesSize(elements,&patch_numrows, &patch_numcols,enum_type);

	/*Allocate matrix: */
	patches=(double*)xmalloc(patch_numrows*patch_numcols*sizeof(double));

	/*Fill patches with NaN, default value: */
	for(i=0;i<patch_numrows;i++){
		for(j=0;j<patch_numcols;j++){
			*(patches+patch_numcols*i+j)=NAN;
		}
	}

	/*Now, go through elements, their inputs with the correct enum_type, and fill the patches: */
	InputToPatches(elements,patches,patch_numrows,patch_numcols);

	/*Create result object embedding patches: */
	result=new DoubleMatResult(id,enum_type,time,step,patches,patch_numrows,patch_numcols);

	/*Assign output pointer*/
	*presult=result;

}
