/*!\file:  ProcessResults.cpp
 * \brief: process patch that was created by an input, for results purposes.
 * For example, velocities need to be in m/yr, melting rates in m/yr, etc ...
 * This centralizes all post-processing of inputs when they are being output to 
 * patched results.
 *
 */ 

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "./InputLocal.h"
#include "../../DataSet/DataSet.h"
#include "../../objects/objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"

void ProcessResults(double* patch, int patch_size,int enum_type,Parameters* parameters){

	int i;
	double yts;

	/*Gete some values from parameters, that are often needed: */
	parameters->FindParam(&yts,YtsEnum);

	switch(enum_type){
		case VxEnum: for(i=0;i<patch_size;i++)patch[i]=patch[i]/yts;break;
		case VyEnum: for(i=0;i<patch_size;i++)patch[i]=patch[i]/yts;break;
		case VzEnum: for(i=0;i<patch_size;i++)patch[i]=patch[i]/yts;break;
		case MeltingRateEnum: for(i=0;i<patch_size;i++)patch[i]=patch[i]/yts;break;
		default: break;
	}
}
