/*!\file DoubleMatResult.c
 * \brief: implementation of the DoubleMatResult object
 */

/*header files: */
/*{{{1*/
#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "stdio.h"
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../DataSet/DataSet.h"
#include "../../include/include.h"
/*}}}*/

/*Object constructors and destructor*/
/*FUNCTION DoubleMatResult::DoubleMatResult(){{{1*/
DoubleMatResult::DoubleMatResult(){
	return;
}
/*}}}*/
/*FUNCTION DoubleMatResult::DoubleMatResult(int id, int enum_type,double time, int step, double* value,int M,int N);{{{1*/
DoubleMatResult::DoubleMatResult(int in_id, int in_enum_type,double in_time, int in_step, double* in_value,int in_M,int in_N){

	id=in_id;
	enum_type=in_enum_type;
	time=in_time;
	step=in_step;

	M=in_M;
	N=in_N;
	value=(double*)xmalloc(M*N*sizeof(double));
	memcpy(value,in_value,M*N*sizeof(double));
}
/*}}}*/
/*FUNCTION DoubleMatResult::~DoubleMatResult(){{{1*/
DoubleMatResult::~DoubleMatResult(){
	xfree((void**)&value);
	return;
}
/*}}}*/

/*Object methods*/
/*FUNCTION DoubleMatResult::copy{{{1*/
Object* DoubleMatResult::copy() {
	
	return new DoubleMatResult(this->id,this->enum_type,this->time,this->step,this->value,this->M,this->N);

}
/*}}}*/
/*FUNCTION DoubleMatResult::DeepEcho{{{1*/
void DoubleMatResult::DeepEcho(void){

	int i,j;

	printf("DoubleMatResult echo:\n");
	printf("   id:  %i \n",this->id);
	printf("   enum:  %i (%s)\n",this->enum_type,EnumAsString(this->enum_type));
	printf("   time:  %g \n",this->time);
	printf("   step:  %i \n",this->step);
	printf("   matrix size: %ix%i\n",this->M,this->N);
	for(i=0;i<this->M;i++){
		for(i=0;i<this->N;i++){
			printf("%i %i %g\n",i,j,*(this->value+N*i+j));
		}
	}
}
/*}}}*/
/*FUNCTION DoubleMatResult::Demarshall{{{1*/
void  DoubleMatResult::Demarshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   i;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*this time, no need to get enum value, the pointer directly points to the beginning of the 
	 *object data (thanks to DataSet::Demarshall):*/

	memcpy(&id,marshalled_dataset,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(&enum_type,marshalled_dataset,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(&time,marshalled_dataset,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(&step,marshalled_dataset,sizeof(step));marshalled_dataset+=sizeof(step);
	
	/*data: */
	memcpy(&M,marshalled_dataset,sizeof(M));marshalled_dataset+=sizeof(M);
	memcpy(&N,marshalled_dataset,sizeof(N));marshalled_dataset+=sizeof(N);
	value=(double*)xmalloc(M*N*sizeof(double));
	memcpy(value,marshalled_dataset,M*N*sizeof(double));marshalled_dataset+=M*N*sizeof(double);

	/*return: */
	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION DoubleMatResult::Echo {{{1*/
void DoubleMatResult::Echo(void){

	printf("DoubleMatResult echo:\n");
	printf("   id:  %i \n",this->id);
	printf("   enum:  %i (%s)\n",this->enum_type,EnumAsString(this->enum_type));
	printf("   time:  %g \n",this->time);
	printf("   step:  %i \n",this->step);
	printf("   matrix size: %ix%i\n",this->M,this->N);

}
/*}}}*/
/*FUNCTION DoubleMatResult::Enum{{{1*/
int DoubleMatResult::Enum(void){

	return DoubleMatResultEnum;

}
/*}}}*/
/*FUNCTION DoubleMatResult::Id{{{1*/
int    DoubleMatResult::Id(void){ return this->id; }
/*}}}*/
/*FUNCTION DoubleMatResult::Marshall{{{1*/
void  DoubleMatResult::Marshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   enum_value=0;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*get enum value of DoubleMatResult: */
	enum_value=DoubleMatResultEnum;
	
	/*marshall enum: */
	memcpy(marshalled_dataset,&enum_value,sizeof(enum_value));marshalled_dataset+=sizeof(enum_value);
	
	/*marshall DoubleMatResult data: */
	memcpy(marshalled_dataset,&id,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(marshalled_dataset,&enum_type,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(marshalled_dataset,&time,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(marshalled_dataset,&step,sizeof(step));marshalled_dataset+=sizeof(step);
	memcpy(marshalled_dataset,&M,sizeof(M));marshalled_dataset+=sizeof(M);
	memcpy(marshalled_dataset,&N,sizeof(N));marshalled_dataset+=sizeof(N);
	memcpy(marshalled_dataset,value,M*N*sizeof(double));marshalled_dataset+=M*N*sizeof(double);

	*pmarshalled_dataset=marshalled_dataset;
}
/*}}}*/
/*FUNCTION DoubleMatResult::MarshallSize{{{1*/
int   DoubleMatResult::MarshallSize(){
	
	return sizeof(id)+
		sizeof(enum_type)+
		sizeof(time)+
		sizeof(step)+
		sizeof(M)+
		sizeof(N)+
		M*N*sizeof(double)+
		sizeof(int); //sizeof(int) for enum value
}
/*}}}*/
/*FUNCTION DoubleMatResult::MyRank{{{1*/
int    DoubleMatResult::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/
/*FUNCTION DoubleMatResult::UpdateInputsFromVector(double* vector, int name, int type) {{{1*/
void  DoubleMatResult::UpdateInputsFromVector(double* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleMatResult::UpdateInputsFromVector(int* vector, int name, int type) {{{1*/
void  DoubleMatResult::UpdateInputsFromVector(int* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleMatResult::UpdateInputsFromVector(bool* vector, int name, int type) {{{1*/
void  DoubleMatResult::UpdateInputsFromVector(bool* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleMatResult::UpdateInputsFromConstant(double constant, int name) {{{1*/
void  DoubleMatResult::UpdateInputsFromConstant(double constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleMatResult::UpdateInputsFromConstant(int constant, int name) {{{1*/
void  DoubleMatResult::UpdateInputsFromConstant(int constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleMatResult::UpdateInputsFromConstant(bool constant, int name) {{{1*/
void  DoubleMatResult::UpdateInputsFromConstant(bool constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
