/*!\file DoubleVecResult.c
 * \brief: implementation of the DoubleVecResult object
 */

/*header files: */
/*{{{1*/
#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "stdio.h"
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../DataSet/DataSet.h"
#include "../../include/include.h"
/*}}}*/

/*Object constructors and destructor*/
/*FUNCTION DoubleVecResult::DoubleVecResult(){{{1*/
DoubleVecResult::DoubleVecResult(){
	return;
}
/*}}}*/
/*FUNCTION DoubleVecResult::DoubleVecResult(int id, int enum_type,double time, int step, double* value,int M){{{1*/
DoubleVecResult::DoubleVecResult(int in_id, int in_enum_type,double in_time, int in_step, double* in_value,int in_M){

	id=in_id;
	enum_type=in_enum_type;
	time=in_time;
	step=in_step;
	
	M=in_M;
	value=(double*)xmalloc(M*sizeof(double));
	memcpy(value,in_value,M*sizeof(double));
}
/*}}}*/
/*FUNCTION DoubleVecResult::~DoubleVecResult(){{{1*/
DoubleVecResult::~DoubleVecResult(){
	xfree((void**)&value);
	return;
}
/*}}}*/

/*Object methods*/
/*FUNCTION DoubleVecResult::copy{{{1*/
Object* DoubleVecResult::copy() {
	
	return new DoubleVecResult(this->id,this->enum_type,this->time,this->step,this->value,this->M);

}
/*}}}*/
/*FUNCTION DoubleVecResult::DeepEcho{{{1*/
void DoubleVecResult::DeepEcho(void){
	
	int i;

	printf("DoubleVecResult echo:\n");
	printf("   id:  %i \n",this->id);
	printf("   enum:  %i (%s)\n",this->enum_type,EnumAsString(this->enum_type));
	printf("   time:  %g \n",this->time);
	printf("   step:  %i \n",this->step);
	printf("   vector size: %i\n",this->M);
	for(i=0;i<this->M;i++){
		printf("%i %g\n",i,this->value[i]);
	}
}
/*}}}*/
/*FUNCTION DoubleVecResult::Demarshall{{{1*/
void  DoubleVecResult::Demarshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   i;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*this time, no need to get enum value, the pointer directly points to the beginning of the 
	 *object data (thanks to DataSet::Demarshall):*/
	memcpy(&id,marshalled_dataset,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(&enum_type,marshalled_dataset,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(&time,marshalled_dataset,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(&step,marshalled_dataset,sizeof(step));marshalled_dataset+=sizeof(step);
	memcpy(&M,marshalled_dataset,sizeof(M));marshalled_dataset+=sizeof(M);
	value=(double*)xmalloc(M*sizeof(double));
	memcpy(value,marshalled_dataset,M*sizeof(double));marshalled_dataset+=M*sizeof(double);

	/*return: */
	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION DoubleVecResult::Echo {{{1*/
void DoubleVecResult::Echo(void){

	printf("DoubleVecResult echo:\n");
	printf("   id:  %i \n",this->id);
	printf("   enum:  %i (%s)\n",this->enum_type,EnumAsString(this->enum_type));
	printf("   time:  %g \n",this->time);
	printf("   step:  %i \n",this->step);
	printf("   vector size: %i\n",this->M);

}
/*}}}*/
/*FUNCTION DoubleVecResult::Enum{{{1*/
int DoubleVecResult::Enum(void){

	return DoubleVecResultEnum;

}
/*}}}*/
/*FUNCTION DoubleVecResult::Id{{{1*/
int    DoubleVecResult::Id(void){ return this->id; }
/*}}}*/
/*FUNCTION DoubleVecResult::Marshall{{{1*/
void  DoubleVecResult::Marshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   enum_value=0;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*get enum value of DoubleVecResult: */
	enum_value=DoubleVecResultEnum;
	
	/*marshall enum: */
	memcpy(marshalled_dataset,&enum_value,sizeof(enum_value));marshalled_dataset+=sizeof(enum_value);
	
	/*marshall DoubleVecResult data: */
	memcpy(marshalled_dataset,&id,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(marshalled_dataset,&enum_type,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(marshalled_dataset,&time,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(marshalled_dataset,&step,sizeof(step));marshalled_dataset+=sizeof(step);
	memcpy(marshalled_dataset,&M,sizeof(M));marshalled_dataset+=sizeof(M);
	memcpy(marshalled_dataset,value,M*sizeof(double));marshalled_dataset+=M*sizeof(double);

	*pmarshalled_dataset=marshalled_dataset;
}
/*}}}*/
/*FUNCTION DoubleVecResult::MarshallSize{{{1*/
int   DoubleVecResult::MarshallSize(){
	
	return sizeof(id)+
		+sizeof(enum_type)+
		+sizeof(time)+
		+sizeof(step)+
		sizeof(M)
		+M*sizeof(double)
		+sizeof(int); //sizeof(int) for enum value
}
/*}}}*/
/*FUNCTION DoubleVecResult::MyRank{{{1*/
int    DoubleVecResult::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/
/*FUNCTION DoubleVecResult::UpdateInputsFromVector(double* vector, int name, int type) {{{1*/
void  DoubleVecResult::UpdateInputsFromVector(double* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleVecResult::UpdateInputsFromVector(int* vector, int name, int type) {{{1*/
void  DoubleVecResult::UpdateInputsFromVector(int* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleVecResult::UpdateInputsFromVector(bool* vector, int name, int type) {{{1*/
void  DoubleVecResult::UpdateInputsFromVector(bool* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleVecResult::UpdateInputsFromConstant(double constant, int name) {{{1*/
void  DoubleVecResult::UpdateInputsFromConstant(double constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleVecResult::UpdateInputsFromConstant(int constant, int name) {{{1*/
void  DoubleVecResult::UpdateInputsFromConstant(int constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION DoubleVecResult::UpdateInputsFromConstant(bool constant, int name) {{{1*/
void  DoubleVecResult::UpdateInputsFromConstant(bool constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
