/*! \file DoubleVecResult.h 
 *  \brief: header file for DoubleVec result object
 */


#ifndef _DOUBLEVECRESULT_H_
#define _DOUBLEVECRESULT_H_

/*Headers:*/
/*{{{1*/

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#ifdef _SERIAL_
#include <mex.h>
#endif


#include "./Result.h"
#include "../../include/include.h"
#include "../../shared/shared.h"
#include "../../include/include.h"
#include "../../include/include.h"
/*}}}*/

class DoubleVecResult: public Result{

	private: 
		int    id;
		int    enum_type;
		double time;
		int    step;
	
		double* value;
		int M;

	public:
		/*constructors, destructors: {{{1*/
		DoubleVecResult();
		DoubleVecResult(int id, int enum_type,double time, int step, double* value,int M);
		~DoubleVecResult();
		/*}}}*/
		/*Object methods: {{{1*/
		Object* copy();
		void  DeepEcho();
		void  Demarshall(char** pmarshalled_dataset);
		void  Echo();
		int   Enum();
		int   Id();
		void  Marshall(char** pmarshalled_dataset);
		int   MarshallSize();
		int   MyRank();

		void  UpdateInputsFromVector(double* vector, int name, int type);
		void  UpdateInputsFromVector(int* vector, int name, int type);
		void  UpdateInputsFromVector(bool* vector, int name, int type);
		void  UpdateInputsFromConstant(double constant, int name);
		void  UpdateInputsFromConstant(int constant, int name);
		void  UpdateInputsFromConstant(bool constant, int name);
		void  UpdateInputsFromSolution(double* solution, int analysis_type, int sub_analysis_type){ISSMERROR("Not implemented yet!");}
		/*}}}*/
		/*Result methods: {{{1*/
		/*}}}*/
};
#endif  /* _DOUBLEVECRESULT_H */
