/*!\file StringResult.c
 * \brief: implementation of the StringResult object
 */

/*header files: */
/*{{{1*/
#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "stdio.h"
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../DataSet/DataSet.h"
#include "../../include/include.h"
/*}}}*/

/*Object constructors and destructor*/
/*FUNCTION StringResult::StringResult(){{{1*/
StringResult::StringResult(){
	return;
}
/*}}}*/
/*FUNCTION StringResult::StringResult(int id, int enum_type,double time, int step, char* value){{{1*/
StringResult::StringResult(int in_id, int in_enum_type,double in_time, int in_step, char* in_value){

	id=in_id;
	enum_type=in_enum_type;
	time=in_time;
	step=in_step;
	value=(char*)xmalloc((strlen(in_value)+1)*sizeof(char));
	strcpy(value,in_value);
	
}
/*}}}*/
/*FUNCTION StringResult::~StringResult(){{{1*/
StringResult::~StringResult(){
	xfree((void**)&value);
}
/*}}}*/

/*Object methods*/
/*FUNCTION StringResult::copy{{{1*/
Object* StringResult::copy() {
	
	return new StringResult(this->id,this->enum_type,this->time,this->step,this->value);

}
/*}}}*/
/*FUNCTION StringResult::DeepEcho{{{1*/
void StringResult::DeepEcho(void){

	printf("StringResult:\n");
	printf("   id:  %i \n",this->id);
	printf("   enum:  %i (%s)\n",this->enum_type,EnumAsString(this->enum_type));
	printf("   time:  %g \n",this->time);
	printf("   step:  %i \n",this->step);
	printf("   string: %s\n",this->value);
}
/*}}}*/
/*FUNCTION StringResult::Demarshall{{{1*/
void  StringResult::Demarshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   i;
	int   stringsize;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*this time, no need to get enum type, the pointer directly points to the beginning of the 
	 *object data (thanks to DataSet::Demarshall):*/
	memcpy(&id,marshalled_dataset,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(&enum_type,marshalled_dataset,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(&time,marshalled_dataset,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(&step,marshalled_dataset,sizeof(step));marshalled_dataset+=sizeof(step);

	memcpy(&stringsize,marshalled_dataset,sizeof(int));marshalled_dataset+=sizeof(int);
	this->value=(char*)xmalloc(stringsize*sizeof(char));
	memcpy(this->value,marshalled_dataset,stringsize*sizeof(char));marshalled_dataset+=stringsize*sizeof(char);

	/*return: */
	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION StringResult::Echo {{{1*/
void StringResult::Echo(void){
	this->DeepEcho();
}
/*}}}*/
/*FUNCTION StringResult::Enum{{{1*/
int StringResult::Enum(void){

	return StringResultEnum;

}
/*}}}*/
/*FUNCTION StringResult::Id{{{1*/
int    StringResult::Id(void){ return this->id; }
/*}}}*/
/*FUNCTION StringResult::Marshall{{{1*/
void  StringResult::Marshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   enum_value=0;
	int   stringsize;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*get enum value of StringResult: */
	enum_value=StringResultEnum;
	
	/*marshall enum: */
	memcpy(marshalled_dataset,&enum_value,sizeof(enum_value));marshalled_dataset+=sizeof(enum_value);

	/*marshall data: */
	memcpy(marshalled_dataset,&id,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(marshalled_dataset,&enum_type,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(marshalled_dataset,&time,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(marshalled_dataset,&step,sizeof(step));marshalled_dataset+=sizeof(step);
	
	stringsize=strlen(this->value)+1;
	memcpy(marshalled_dataset,&stringsize,sizeof(stringsize));marshalled_dataset+=sizeof(stringsize);
	memcpy(marshalled_dataset,this->value,stringsize*sizeof(char));marshalled_dataset+=stringsize*sizeof(char);

	*pmarshalled_dataset=marshalled_dataset;
}
/*}}}*/
/*FUNCTION StringResult::MarshallSize{{{1*/
int   StringResult::MarshallSize(){

	int stringsize;
	stringsize=strlen(this->value)+1;
	
	return sizeof(id)+
		sizeof(enum_type)+
		sizeof(time)+
		sizeof(step)+
		sizeof(int)+
		stringsize*sizeof(char)+
		sizeof(int); //sizeof(int) for enum value
}
/*}}}*/
/*FUNCTION StringResult::MyRank{{{1*/
int    StringResult::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/
/*FUNCTION StringResult::UpdateInputsFromVector(double* vector, int name, int type) {{{1*/
void  StringResult::UpdateInputsFromVector(double* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION StringResult::UpdateInputsFromVector(int* vector, int name, int type) {{{1*/
void  StringResult::UpdateInputsFromVector(int* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION StringResult::UpdateInputsFromVector(bool* vector, int name, int type) {{{1*/
void  StringResult::UpdateInputsFromVector(bool* vector, int name, int type){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION StringResult::UpdateInputsFromConstant(double constant, int name) {{{1*/
void  StringResult::UpdateInputsFromConstant(double constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION StringResult::UpdateInputsFromConstant(int constant, int name) {{{1*/
void  StringResult::UpdateInputsFromConstant(int constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/
/*FUNCTION StringResult::UpdateInputsFromConstant(bool constant, int name) {{{1*/
void  StringResult::UpdateInputsFromConstant(bool constant, int name){
	/*Nothing updated yet*/
}
/*}}}*/

/*StringResult methods: */
