
/*!\file: ControlRestart.cpp
 * \brief: core of the control solution 
 */ 

#include "./solutions.h"
#include "../modules/modules.h"
#include "../EnumDefinitions/EnumDefinitions.h"

void ControlRestart(Model* model,double* param_g){

	extern int my_rank;

	/*output: */
	Results* results=NULL;
	char*    outputfilename=NULL;

	/*Intermediary: */
	int      i;
	int      numberofnodes;
	double* param_g_copy;

	/*Recover parameters used throughout the solution:*/
	model->FindParam(&numberofnodes,NumberOfNodesEnum);
	model->FindParam(&outputfilename,OutputFileNameEnum);

	/*Plug COPYS of the results into output dataset: 
	 * only the pointer is given to temporary_results and at the
	 * end of ProcessResultsx the pointer is deleted. That would 
	 * destroy param_g*/

	param_g_copy=(double*)xcalloc(numberofnodes,sizeof(double));
	for(i=0;i<numberofnodes;i++) param_g_copy[i]=param_g[i];

	results=new Results();
	results->AddObject(new Result(results->Size()+1,0,1,"param_g",param_g_copy,numberofnodes));
	results->AddObject(new StringResult(results->Size()+1,AnalysisTypeEnum,0,1,EnumAsString(DiagnosticAnalysisEnum)));

	//Write results on disk
	OutputResults(results,outputfilename);
	
	/*Free ressources:*/
	delete results;
	xfree((void**)&outputfilename);
	xfree((void**)&param_g_copy);
}
